@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: remote.c 7864 2009-02-10 16:21:57Z charles $
+ * $Id: remote.c 7969 2009-02-27 15:58:08Z charles $
  */
 
 #include <errno.h>
@@ -15,11 +15,11 @@
 #include <stdlib.h>
 #include <string.h> /* strcmp */
 
-#ifdef WIN32 
- #include <direct.h> /* getcwd */ 
-#else 
- #include <unistd.h> /* getcwd */ 
-#endif 
+#ifdef WIN32
+ #include <direct.h> /* getcwd */
+#else
+ #include <unistd.h> /* getcwd */
+#endif
 
 #include <libevent/event.h>
 
@@ -126,18 +126,18 @@ static int    debug = 0;
 static char * auth = NULL;
 static char * netrc = NULL;
 
-static char* 
-tr_getcwd( void ) 
-{ 
-    char buf[2048]; 
-    *buf = '\0'; 
-#ifdef WIN32 
-    _getcwd( buf, sizeof( buf ) ); 
-#else 
-    getcwd( buf, sizeof( buf ) ); 
-#endif 
-    return tr_strdup( buf ); 
-} 
+static char*
+tr_getcwd( void )
+{
+    char buf[2048];
+    *buf = '\0';
+#ifdef WIN32
+    _getcwd( buf, sizeof( buf ) );
+#else
+    getcwd( buf, sizeof( buf ) );
+#endif
+    return tr_strdup( buf );
+}
 
 static char*
 absolutify( const char * path )
@@ -267,6 +267,8 @@ static const char * details_keys[] = {
 };
 
 static const char * list_keys[] = {
+    "error",
+    "errorString",
     "eta",
     "id",
     "leftUntilDone",
@@ -454,8 +456,8 @@ readargs( int           argc,
                 break;
 
             case 'V':
-		fprintf(stderr, "Transmission %s\n", LONG_VERSION_STRING);
-		exit(0);
+                fprintf( stderr, "Transmission %s\n", LONG_VERSION_STRING );
+                exit( 0 );
                 break;
 
             case 'w': {
@@ -729,8 +731,9 @@ getStatusString( tr_benc * t, char * buf, size_t buflen )
                     tr_strlcpy( buf, "Uploading", buflen );
                 else
                     tr_strlcpy( buf, "Seeding", buflen );
-            } else
+            } else {
                 tr_strlcpy( buf, "Idle", buflen );
+            }
             break;
         }
     }
@@ -792,7 +795,6 @@ printSession( tr_benc * top )
             printf( "  Uploadlimit enabled:   %s\n", ( i ? "Yes" : "No" ) );
         if( tr_bencDictFindInt( args, "speed-limit-up", &i ) )
             printf( "  Uploadlimit:   %6" PRId64 " KB/sec\n", i );
-		
     }
 }
 
@@ -1102,7 +1104,10 @@ printTorrentList( tr_benc * top )
       && ( tr_bencDictFindList( args, "torrents", &list ) ) )
     {
         int i, n;
-        printf( "%-4s  %-4s  %9s  %-8s  %6s  %6s  %-5s  %-11s  %s\n",
+        int64_t total_up = 0, total_down = 0, total_size = 0;
+        char haveStr[32];
+
+        printf( "%-4s   %-4s  %9s  %-8s  %6s  %6s  %-5s  %-11s  %s\n",
                 "ID", "Done", "Have", "ETA", "Up", "Down", "Ratio", "Status",
                 "Name" );
         for( i = 0, n = tr_bencListSize( list ); i < n; ++i )
@@ -1125,8 +1130,9 @@ printTorrentList( tr_benc * top )
                 char etaStr[16];
                 char statusStr[64];
                 char ratioStr[32];
-                char haveStr[32];
                 char doneStr[8];
+                int64_t error;
+                char errorMark;
 
                 if( sizeWhenDone )
                     tr_snprintf( doneStr, sizeof( doneStr ), "%d%%", (int)( 100.0 * ( sizeWhenDone - leftUntilDone ) / sizeWhenDone ) );
@@ -1139,9 +1145,13 @@ printTorrentList( tr_benc * top )
                     etaToString( etaStr, sizeof( etaStr ), eta );
                 else
                     tr_snprintf( etaStr, sizeof( etaStr ), "Done" );
+                if( tr_bencDictFindInt( d, "error", &error ) && error )
+                    errorMark = '*';
+                else
+                    errorMark = ' ';
                 printf(
-                    "%4d  %4s  %9s  %-8s  %6.1f  %6.1f  %5s  %-11s  %s\n",
-                    (int)id,
+                    "%4d%c  %4s  %9s  %-8s  %6.1f  %6.1f  %5s  %-11s  %s\n",
+                    (int)id, errorMark,
                     doneStr,
                     haveStr,
                     etaStr,
@@ -1150,8 +1160,17 @@ printTorrentList( tr_benc * top )
                     strlratio2( ratioStr, ratio, sizeof( ratioStr ) ),
                     getStatusString( d, statusStr, sizeof( statusStr ) ),
                     name );
+
+                total_up += up;
+                total_down += down;
+                total_size += sizeWhenDone - leftUntilDone;
             }
         }
+
+        printf( "Sum:         %9s             %6.1f  %6.1f\n",
+                strlsize( haveStr, total_size, sizeof( haveStr ) ),
+                total_up / 1024.0,
+                total_down / 1024.0 );
     }
 }
 
