@@ -4,22 +4,29 @@
 #include "peer-msgs.h"
 #include "utils.h"
 
-#define VERBOSE 0
+#undef VERBOSE
 
-#define check( A ) \
+#ifdef VERBOSE
+  #define check( A ) \
     { \
         ++test; \
         if( A ){ \
-            if( VERBOSE ) \
-                fprintf( stderr, "PASS test #%d (%s, %d)\n", test, __FILE__,\
-                         __LINE__ );\
+            fprintf( stderr, "PASS test #%d (%s, %d)\n", test, __FILE__, __LINE__ ); \
         } else { \
-            if( VERBOSE ) \
-                fprintf( stderr, "FAIL test #%d (%s, %d)\n", test, __FILE__,\
-                         __LINE__ );\
+            fprintf( stderr, "FAIL test #%d (%s, %d)\n", test, __FILE__, __LINE__ ); \
             return test; \
         } \
     }
+#else
+  #define check( A ) \
+    { \
+        ++test; \
+        if( !( A ) ){ \
+            fprintf( stderr, "FAIL test #%d (%s, %d)\n", test, __FILE__, __LINE__ ); \
+            return test; \
+        } \
+    }
+#endif
 
 int
 main( void )
@@ -27,7 +34,7 @@ main( void )
     uint32_t           i;
     int                test = 0;
     uint8_t            infohash[SHA_DIGEST_LENGTH];
-    tr_address         addr;
+    struct tr_address  addr;
     tr_piece_index_t   pieceCount = 1313;
     size_t             numwant;
     size_t             numgot;
@@ -36,7 +43,7 @@ main( void )
 
     for( i = 0; i < SHA_DIGEST_LENGTH; ++i )
         infohash[i] = 0xaa;
-    tr_netResolve( "80.4.4.200", &addr );
+    tr_pton( "80.4.4.200", &addr );
 
     numwant = 7;
     numgot = tr_generateAllowedSet( buf, numwant, pieceCount, infohash, &addr );
